package com.persagy.bdtp.adm.controller;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.bdtp.adm.client.RwdClient;
import com.persagy.bdtp.adm.common.AdmConst;
import com.persagy.bdtp.adm.common.AdmRequest;
import com.persagy.bdtp.adm.common.AdmResult;
import com.persagy.bdtp.adm.dao.AdmPipeConfigMapper;
import com.persagy.bdtp.adm.dao.AdmRelsConfigMapper;
import com.persagy.bdtp.adm.entity.*;
import com.persagy.bdtp.adm.service.IConfigService;
import com.persagy.bdtp.adm.service.ISyncApp;
import com.persagy.bdtp.adm.service.ISyncOldAdm;
import com.persagy.bdtp.adm.service.ServiceUtil;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.constant.ResponseCode;
import com.persagy.dmp.common.exception.BusinessException;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.define.entity.GraphDefine;
import com.persagy.dmp.define.entity.RelationDefine;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.*;

@CrossOrigin
@RestController
@RequestMapping("/tool")
public class ToolController {

	@Autowired
	private ObjectMapper objectMapper;

	@Autowired
	private ISyncApp syncApp;

	@Autowired
	private RwdClient rwdClient;

	@Autowired
	private IConfigService configService;

	@RequestMapping("/dict")
	public AdmResult<Object> dict(@RequestBody AdmRequest req){
		Map<String,Object> map = syncApp.queryDict(req.getGroupCode(), req.getProjectId(), req.getUserId());

		List<Map<String, Object>> majorList = (List)map.get("major");
		List<TypeDef> sysList = (List)map.get("system");
		List<TypeDef> equipList = (List)map.get("equipment");
		distinct(sysList);
		distinct(equipList);

		clearTime(majorList);
		clearTime(sysList);
		clearTime(equipList);

		//组织树形结构
		for(Map<String, Object> major : majorList){
			ArrayList<TypeDef> childrenSys = new ArrayList<>();
			major.put("children", childrenSys);
			String majorCode = (String) major.get("code");
			for(Iterator<TypeDef> sysIter = sysList.iterator(); sysIter.hasNext();){
				TypeDef sysDef = sysIter.next();
				if(sysDef.getMajorCode() != null && sysDef.getMajorCode().equals(majorCode)){
					childrenSys.add(sysDef);
					sysIter.remove();
				}
			}
		}
		map.remove("system");

		List<RelationDefine> relDefs = (List<RelationDefine>)map.get("relation");
		List<GraphDefine> graph = ServiceUtil.call (() -> rwdClient.queryGraphDef(req.getGroupCode(), req.getProjectId(), AdmConst.APP_ID, req.getUserId(), new QueryCriteria()));
		List<Map<String, Object>> rels = new ArrayList<>();
		for(GraphDefine gd : graph){
			HashMap<String, Object> m = new HashMap<>();
			m.put("code", gd.getCode());
			m.put("name", gd.getName());
			m.put("type", "graph");
			rels.add(m);

			ArrayList<RelationDefine> chidren = new ArrayList<>();
			m.put("children", chidren);
			for(RelationDefine relDef : relDefs) {
				if(relDef.getGraphCode().equals(gd.getCode()))
					chidren.add(relDef);
			}
		}
		map.put("relation", rels);

		return AdmResult.success(map);
	}

	private void distinct(List<TypeDef> list){
		List<TypeDef> result = new ArrayList<>();
		ArrayList<String> codes = new ArrayList<>();
		for(TypeDef def : list) {
			if(!codes.contains(def.getCode())) {
				codes.add(def.getCode());
				result.add(def);
			}
		}
		list.clear();
		list.addAll(result);
	}

	private void clearTime(List<?> list){
		for(Object o : list) {
			if(o instanceof TypeDef) {
				((TypeDef) o).setCreationTime(null);
				((TypeDef) o).setModifiedTime(null);
			} else if(o instanceof Map) {
				((Map) o).put("creationTime", null);
				((Map) o).put("modifiedTime", null);
				((Map) o).put("name", ((Map) o).get("label")); //handle major
			}
		}
	}

	@RequestMapping("/cfgs")
	public AdmResult<Object> cfgs(@RequestBody AdmRequest req){
		String projectId = req.getProjectId();

		//配置数据查询
		List<AdmRelsConfig> relConfig = configService.queryRelsConfig(projectId);
		List<AdmPipeConfig> pipeConfig = configService.queryPipeConfig(projectId);

		HashMap<String, Object> data = new HashMap<>();
		data.put("relConfig", relConfig);
		data.put("pipeConfig", pipeConfig);

		return AdmResult.success(data);
	}

	@Autowired
	private AdmRelsConfigMapper relsConfigMapper;
	@Autowired
	private AdmPipeConfigMapper pipeConfigMapper;
	@PostMapping("/updateCfgItem")
	public AdmResult<Object> updateCfgItem(@RequestBody Map<String, Object> content) {
		String type = (String) content.get("type");
		Map<String, Object> itemMap = (Map<String, Object>) content.get("item");
		String delId = (String) content.get("delId");
		String newId = null;
		if(itemMap != null && !itemMap.containsKey("id")){
			newId = IdUtil.fastSimpleUUID();
			itemMap.put("id", newId);
		}

		if("rel".equals(type)) {
			if(StrUtil.isNotBlank(delId))
				relsConfigMapper.deleteById(delId);
			else {
				AdmRelsConfig cfg = mapToObj(itemMap, AdmRelsConfig.class);
				if(newId != null)
					relsConfigMapper.insert(cfg);
				else
					relsConfigMapper.updateById(cfg);
			}
		} else if ("pipe".equals(type)) {
			if(StrUtil.isNotBlank(delId))
				pipeConfigMapper.deleteById(delId);
			else {
				AdmPipeConfig cfg = mapToObj(itemMap, AdmPipeConfig.class);
				if(newId != null)
					pipeConfigMapper.insert(cfg);
				else
					pipeConfigMapper.updateById(cfg);
			}
		}

		return AdmResult.success(newId);
	}

	private <T extends BaseAdmEntity> T mapToObj(Map<String, Object> map, Class<T> cls){
		try {
			String json = objectMapper.writeValueAsString(map);
			return objectMapper.readValue(json, cls);
		} catch (IOException e) {
			e.printStackTrace();
			throw new BusinessException(ResponseCode.A0427.getCode(), ResponseCode.A0427.getDesc());
		}
	}

}
